;																		
; VIDEOSIGNAL / VOLLGRAFIK / BAS / FBAS /TV-GRAFIC 						
;																		
; MINIMALVARIANTE														
; 																		
; 320x256 PIXEL															
;																		
; ATmega1284P-20MHz														
;																		
; AVR-STUDIO 4															
; Bernhard.Erfurt@gmx.de												
; VERSION: 1a (11/2014)													
;																		

;																		
; Dieses Assembler-Programm generiert ein Fernsehbild.					
;																		
; Vollrafik ist mglich.												
;																		
; Der Bildspeicher ergibt sich aus 320*256=81.920PIXEL=10.240 BYTE		
;																		

;																		
; Das TV-Bild generiert ein 64s TIMER (TIMER-0),						
; als Kontrolltimer fr die Taktkorrektur dient der TIMER-2.			
;																		
; Jede der 625 TV-Zeilen wird separat in der "TIM0_COMPA"generiert und 	
; die TV-Norm eingehalten.												
;																		

;																		
; Um wertvolle Takte in der Interruptroutine zu sparen, wird die Zeit 	
; der hinteren Schwarzschulter genutzt, um jede							
; TV-Bild-Daten_Zeile auf Daten zu prfen, liegen keine DATEN vor,		
; dann wird diese Zeile nicht geschrieben (Zeile bleibt Dunkel)			
; Liege nur Daten in einzelnen ZEILEN-Bereichen vor, 					
; wird u.U. nur ein Teil der TV-Zeile geschrieben, das spart Takte !!!	
;																		

;																		
;Nicht alle nutzbaren TV-Zeilen werden abgebildet, ca. 60 TV-ZEILEN		
;bleiben ungenutzt.														
;																		

;																		
; Die Datenausgabe (horizontale Pixel) generiert der PORT-A als 		
; Schieberegister, ist aber	dadurch komplett belegt, 					
; der PORT kann aber bei Bedarf abgendert werden.						
;																		


;																		
; Kurz nach Programmstart synchronisieren sich TIMER-0 und TIMER-1		
; damit der Bildaufbau jitterfrei erfolgen kann.						
;																		

;																		
; GRN leuchtet: alles ok und alles schn								
; GELB leuchtet: 														
; ROT  leuchtet: 														
; - Programm luft, wenn mglich mit Error weiter						
; - die automatische Taktkorrektur im Interrupt versagt (TIM0_COMPA)	
; - die Zeilenzuordnung stimmt nicht					(TIM0_COMPA)	
; - diverse andere Fehlern												
; - ERROR-CODE lsst sich ggf. anzeigen									
;																		

;																		
; Blinken die LEDs, das ist nicht gut !									
; ROT blinkt: 				Fehler im Programmcheck						
; ROT+GELB blinken: 		WDR-Zeitberschreitung						
; ROT+GELB+GRN blinken: 	unkorrekter Interruptaufruf					
;																		


;												
; Anschlussbelegung PINS						
;												
; PA0 ==> OUT-DATA s/w (0,3V-1V)				
; PA1 ==> 										
; PA2 ==> 										
; PA3 ==> 										
; PA4 ==> 										
; PA5 ==> 										
; PA6 ==> 										
; PA7 ==> 										
;												
; PB0 ==> OUT SYNCHRONIMPULSE (0V-0,3V)			
; PB1 ==> 										
; PB2 ==> 										
; PB3 ==> OUT TESTAUSGANG (15625/2=7.812,5 Hz)	
; PB4 ==> 										
; PB5 ==> 										
; PB6 ==> 										
; PB7 ==> 										
;												
; PC0 ==> 										
; PC1 ==> 										
; PC2 ==> 										
; PC3 ==> 										
; PC4 ==>  										
; PC5 ==> LED GRN								
; PC6 ==> LED GELB								
; PC7 ==> LED ROT								
;												
; PD0 ==> 										
; PD1 ==> 										
; PD2 ==> 										
; PD3 ==> 										
; PD4 ==> 										
; PD5 ==>  										
; PD6 ==> 										
; PD7 ==> 										
;												

; ATmega1284P 									
.include "m1284pdef.inc"
; KONSTANTEN									
.equ TAKT							=20000000	; 20 MHz	
; TIMER-WERTE									
.equ TIMER0_CTC_FREQUENZ			=15625/2
.equ TIMER0_N						= 8
.equ TIMER0_CTC						=(TAKT/(2*TIMER0_N*TIMER0_CTC_FREQUENZ))-1
.equ TIMERWERT1						=65536-31250;(2 hoch 16 minus...) bei 20MHz + Vorteiler 64 => 100ms 		
; LED-GRUEN										
.equ LED_GRUEN_PIN_NR				= 5
.equ LED_GRUEN_DDR					= DDRC
.equ LED_GRUEN_PORT					= PORTC
; LED-GELB										
.equ LED_GELB_PIN_NR				= 6
.equ LED_GELB_DDR					= DDRC
.equ LED_GELB_PORT					= PORTC
; LED-ROT										
.equ LED_ROT_PIN_NR					= 7
.equ LED_ROT_DDR					= DDRC
.equ LED_ROT_PORT					= PORTC
; TV-DATA										
.equ TV_DATA_PIN_NR					= 0
.equ TV_DATA_DDR					= DDRA
.equ TV_DATA_PORT					= PORTA
; TV-SYNCHRON									
.equ TV_SYNC_PIN_NR					= 0
.equ TV_SYNC_DDR					= DDRB
.equ TV_SYNC_PORT					= PORTB
; ERROR CODES									
.equ ERROR_CODE_TV_A				=10	; TAKTKORREKTUR				
.equ ERROR_CODE_TV_B				=11	; SPRUNGWEITE				
.equ ERROR_CODE_TV_C				=12	; ZEILENBERWACHUNG			
.equ ERROR_CODE_TV_D				=13	; ZEILENBERWACHUNG			
; REGISTER												
.def SREG_KOPIE		= R6	; BILDERZEIGUNG - nutzt nur der 64s-TIMER-Interrupt !
.def temp_KOPIE		= R7	; BILDERZEIGUNG - nutzt nur der 64s-TIMER-Interrupt !
.def temp1_KOPIE	= R8	; BILDERZEIGUNG - nutzt nur der 64s-TIMER-Interrupt !
.def temp2_KOPIE	= R9	; BILDERZEIGUNG - nutzt nur der 64s-TIMER-Interrupt !
.def ZL_KOPIE		= R10	; BILDERZEIGUNG - nutzt nur der 64s-TIMER-Interrupt !
.def ZH_KOPIE		= R11	; BILDERZEIGUNG - nutzt nur der 64s-TIMER-Interrupt !
.def TV_ZEILE_NR_L 	= R12	; BILDERZEIGUNG - nutzt nur der 64s-TIMER-Interrupt !
.def TV_ZEILE_NR_H 	= R13	; BILDERZEIGUNG - nutzt nur der 64s-TIMER-Interrupt !
.def NULL 			= R14	; ist immer NULL 	
.def EINS 			= R15	; ist immer EINS 	
.def temp 			= R16
.def temp1 			= R17
.def temp2 			= R18
.def temp3 			= R19
.def temp4 			= R20
.def temp5 			= R21
.def temp6 			= R22
.def temp7 			= R23
.def temp8 			= R24
.def temp9 			= R25

; SRAM-BELEGUNG Adr:0x0100-0x40FF(256-16.639=16383)		
.equ adr_DATEN						=SRAM_START		

.equ adr_ERROR_CODE_TV				=SRAM_START+0		; ERROR CODE TV				
.equ adr_ERROR_WERT_TV				=SRAM_START+1		; ERROR CODE TV (WERT)		

.equ adr_TV_ZEIGER_DATA_L			=SRAM_START+800		; ZEIGER fr BILD-DATEN		
.equ adr_TV_ZEIGER_DATA_H			=SRAM_START+801

.equ adr_TV_BILDBEREICH				=SRAM_START+1000 	;(40*256=10240 Bytes)		
.equ adr_TV_BILDBEREICH_ENDE		=SRAM_START+1000-1+40*256

.equ adr_SRAM_CHECK					=SRAM_START+15500	; SRAM CHECK muss immer "S" sein	



.cseg						;Beginn eines Code-Segmentes
.org 0						;Startadresse=0				
.org 0x0000 rjmp RESET 			 ; Reset									
.org 0x0002 rjmp INTERRUPT_ERROR ;rjmp INT0 ; IRQ0							
.org 0x0004 rjmp INTERRUPT_ERROR ;rjmp INT1 ; IRQ1							
.org 0x0006 rjmp INTERRUPT_ERROR ;rjmp INT2 ; IRQ2							
.org 0x0008 rjmp INTERRUPT_ERROR ;rjmp PCINT0 ; PCINT0						
.org 0x000A rjmp INTERRUPT_ERROR ;rjmp PCINT1 ; PCINT1						
.org 0x000C rjmp INTERRUPT_ERROR ;rjmp PCINT2 ; PCINT2						
.org 0x000E rjmp INTERRUPT_ERROR ;rjmp PCINT3 ; PCINT3						
.org 0x0010 jmp WDT 		; Watchdog Timeout								
.org 0x0012 rjmp INTERRUPT_ERROR ;rjmp TIM2_COMPA ; Timer2 CompareA			
.org 0x0014 rjmp INTERRUPT_ERROR ;rjmp TIM2_COMPB ; Timer2 CompareB			
.org 0x0016 rjmp INTERRUPT_ERROR ;rjmp TIM2_OVF 	; Timer2 Overflow		
.org 0x0018 rjmp INTERRUPT_ERROR ;rjmp TIM1_CAPT ; Timer1 Capture			
.org 0x001A rjmp INTERRUPT_ERROR ;rjmp TIM1_COMPA ; Timer1 CompareA			
.org 0x001C rjmp INTERRUPT_ERROR ;rjmp TIM1_COMPB ; Timer1 CompareB			
.org 0x001E rjmp INTERRUPT_ERROR ;TIM1_OVF 	; Timer1 Overflow				
.org 0x0020 rjmp TIM0_COMPA ; Timer0 CompareA								
.org 0x0022 rjmp INTERRUPT_ERROR ;rjmp TIM0_COMPB ; Timer0 CompareB			
.org 0x0024 rjmp INTERRUPT_ERROR ;rjmp TIM0_OVF 	; Timer0 Overflow		
.org 0x0026 rjmp INTERRUPT_ERROR ;rjmp SPI_STC ; SPI Transfer Complete		
.org 0x0028 rjmp INTERRUPT_ERROR ;rjmp USART0_RXC ; USART0 RX Complete		
.org 0x002A rjmp INTERRUPT_ERROR ;rjmp USART0_UDRE ; USART0,UDR Empty		
.org 0x002C rjmp INTERRUPT_ERROR ;rjmp USART0_TXC ; USART0 TX Complete		
.org 0x002E rjmp INTERRUPT_ERROR ;rjmp ANA_COMP ; Analog Comparator			
.org 0x0030 rjmp INTERRUPT_ERROR ;rjmp ADC ; ADC Conversion Complete		
.org 0x0032 rjmp INTERRUPT_ERROR ;rjmp EE_RDY ; EEPROM Ready				
.org 0x0034 rjmp INTERRUPT_ERROR ;rjmp TWI ; 2-wire Serial					
.org 0x0036 rjmp INTERRUPT_ERROR ;rjmp SPM_RDY ; SPM Ready					
.org 0x0038 rjmp INTERRUPT_ERROR ;rjmp USART1_RXC ; USART1 RX Complete		
.org 0x003A rjmp INTERRUPT_ERROR ;rjmp USART1_UDRE ; USART1,UDR Empty		
.org 0x003C rjmp INTERRUPT_ERROR ;rjmp USART1_TXC ; USART1 TX Complete		
.org 0x003E rjmp INTERRUPT_ERROR ;rjmp TIM3_CAPT ; Timer3 Capture			
.org 0x0040 rjmp INTERRUPT_ERROR ;rjmp TIM3_COMPA ; Timer3 CompareA			
.org 0x0042 rjmp INTERRUPT_ERROR ;rjmp TIM3_COMPB ; Timer3 CompareB			
.org 0x0044 rjmp INTERRUPT_ERROR ;rjmp TIM3_OVF 	; Timer3 Overflow		
; Unkorrekten Interrupt Aufruf 						
INTERRUPT_ERROR:
	CLI											; INTERRUPTS sperren	
	cbi (LED_GRUEN_PORT),(LED_GRUEN_PIN_NR)		; GRN an   
	cbi (LED_GELB_PORT),(LED_GELB_PIN_NR)		; GELB an	
	cbi (LED_ROT_PORT),(LED_ROT_PIN_NR)			; ROT  an	
	call wait_100ms
	sbi (LED_GRUEN_PORT),(LED_GRUEN_PIN_NR)		; GRN aus	
	sbi (LED_GELB_PORT),(LED_GELB_PIN_NR)		; GELB aus  
	sbi (LED_ROT_PORT),(LED_ROT_PIN_NR)			; ROT  aus	
	call wait_100ms
	rjmp INTERRUPT_ERROR
;-------------------------------------------------------------------------------
RESET:                        
; STACK initialisieren								
	ldi temp, LOW(RAMEND)
    out SPL, temp
    ldi temp, HIGH(RAMEND)
    out SPH, temp
; SRAM_CLEAR										
	call SRAM_CLEAR
; STARTWERTE										
	; NULL						
	clr NULL
	; EINS						
	ldi temp,1
	mov EINS,temp
	; SRAM CHECK				
	ldi temp,'S'
	STS(adr_SRAM_CHECK),temp
	; TV_ZEILE					
	ldi temp1,LOW (-1)
	ldi temp2,HIGH(-1)
	mov TV_ZEILE_NR_L,temp1
	mov TV_ZEILE_NR_H,temp2
	;3-STRICHE auf dem MONITOR	
	ldi temp,255
	STS(adr_TV_BILDBEREICH+(256*320/16)+20+0),temp
	STS(adr_TV_BILDBEREICH+(256*320/16)+20+80),temp
	STS(adr_TV_BILDBEREICH+(256*320/16)+20+160),temp
; PORT A+B+C+D initialisieren (alles EINGNGE+PULL-UP)		
	ldi temp,255
	out DDRA, NULL
	out PORTA,temp
	out DDRB, NULL
	out PORTB,temp
	out DDRC, NULL
	out PORTC,temp
	out DDRD, NULL
	out PORTD,temp
; LED_INITIALISIERUNG										
	call LED_INITIALISIERUNG
; TV_PIN_INITIALISIERUNG									
	call TV_PIN_INITIALISIERUNG
; WDR_INITIALISIERUNG										
	call WDR_INITIALISIERUNG
; TIMER INITALISIERUNG										
	call TIMER0_INITIALISIERUNG_CTC
	call TIMER2_INITIALISIERUNG
; Interrupts freigeben										
	sei
	cbi (LED_GRUEN_PORT),LED_GRUEN_PIN_NR		; GRN an   
; EINSCHWINGEN												
	call wait_10ms
	sbi (LED_GELB_PORT),LED_GELB_PIN_NR			; GELB aus	
	sbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  aus		
	STS(adr_ERROR_CODE_TV),NULL					; ERROR CLEAR	
	STS(adr_ERROR_WERT_TV),NULL					; ERROR CLEAR	
; ##############################################################################
; ##############################################################################
; ##############################################################################
LOOP:
	WDR										; Watch-Dog-RESET	
	call PROGRAMM_CHECK
rjmp LOOP
; ##############################################################################
; ##############################################################################
; ##############################################################################
.include "TV_WAIT_20MHZ.asm"		; 20 MHz						
.include "TV_TV_MACRO.asm"			; MAKROS						
.include "TV_TV_BILD_TIMER.asm"		; TIMER+BILDAUFBAU				
.include "TV_SONSTIGES.asm"			; sonstiges						




